/*****************************************************************************
 * Test2_MDMA_1Dauto_1_Core1.h
 *****************************************************************************/

#ifndef __TEST2_MDMA_1DAUTO_1_CORE1_H__
#define __TEST2_MDMA_1DAUTO_1_CORE1_H__
/* Add your custom header content here */
/*=============  INCLUDES  =============*/
#include <services/dma/adi_dma.h>
#include <services/spu/adi_spu.h>
#include <services/tru/adi_tru.h>
#include <stdio.h>
#include "adi_initialize.h"
#include <sys/platform.h>
#include "math.h"
#include "stdlib.h"
#include "time.h"

/*=============  DEFINES   =============*/
/* Enable macro to display debug information */
#define ENABLE_DEBUG_INFO

/* Enable this macro to use mdma1DAuto API instead of mdma1D */
#define USE_MDMA1D_AUTO

#define SYSCLK 1000

/* Memory Copy time out value */
#define MEMCOPY_TIME_OUT_VAL        (0x7FFFFFFu)

#define MDMA0_SRC_DMA8_SPU_PID       (137u)
#define MDMA0_DST_DMA9_SPU_PID       (138u)
#define MDMA1_SRC_DMA18_SPU_PID      (139u)
#define MDMA1_DST_DMA19_SPU_PID      (140u)
#define MDMA2_SRC_DMA39_SPU_PID      (126u) /* Enhanced BW DMA 0 */
#define MDMA3_SRC_DMA43_SPU_PID      (127u) /* Max BW DMA 0 */

/* New instances added */
#define MDMA4_SRC_DMA45_SPU_PID      (141u)
#define MDMA4_DST_DMA46_SPU_PID      (142u)
#define MDMA5_SRC_DMA47_SPU_PID      (143u)
#define MDMA5_DST_DMA48_SPU_PID      (144u)
#define MDMA6_SRC_DMA49_SPU_PID      (149u) /* Enhanced BW DMA 1 */
#define MDMA7_SRC_DMA51_SPU_PID      (128u) /* Max BW DMA 1 */

#define SOFT0_TRIGGER_MASTER_ID		 (136u)



#define MDMA0_SRC_TRIGGER_SLAVE_ID		 (4u)
#define MDMA0_DST_TRIGGER_SLAVE_ID		 (5u)
#define MDMA1_SRC_TRIGGER_SLAVE_ID		 (6u)
#define MDMA1_DST_TRIGGER_SLAVE_ID		 (7u)
#define MDMA2_SRC_TRIGGER_SLAVE_ID		 (33u)
#define MDMA2_DST_TRIGGER_SLAVE_ID		 (34u)
#define MDMA3_SRC_TRIGGER_SLAVE_ID		 (35u)
#define MDMA3_DST_TRIGGER_SLAVE_ID		 (36u)
#define MDMA4_SRC_TRIGGER_SLAVE_ID		 (8u)
#define MDMA4_DST_TRIGGER_SLAVE_ID		 (9u)
#define MDMA5_SRC_TRIGGER_SLAVE_ID		 (10u)
#define MDMA5_DST_TRIGGER_SLAVE_ID		 (11u)
#define MDMA6_SRC_TRIGGER_SLAVE_ID		 (37u)
#define MDMA6_DST_TRIGGER_SLAVE_ID		 (38u)
#define MDMA7_SRC_TRIGGER_SLAVE_ID		 (39u)
#define MDMA7_DST_TRIGGER_SLAVE_ID		 (40u)

#define TRU0_SLV0_TRIGGER_SLAVE_ID		 (161u) /* SHARC0 - 161, SHARC1 - 165, ARM - 157 */


/* Maximum transfer size in bytes */
#define MAX_MEMCOPY_BUF_SIZE	16384

/* IF (Debug info enabled) */
#if defined(ENABLE_DEBUG_INFO)
#define DBG_MSG                     printf
#else
#define DBG_MSG(...)
#endif

/* Maximum number of test combinations */
#define MAX_TEST_COMBINATIONS 1000

/* Change the test parameter input file name here if required */
#if 1
	#define TEST_PARAM_INPUT_FILE "testparam.dat"
#else
	#define TEST_PARAM_INPUT_FILE "testparam_cache.dat"
#endif

/* Enumeration for different type of memories */
typedef enum MEMTYPE
{
	L1MEM,
	L2MEM,
	L3MEM
} MEMTYPE;

typedef struct TEST_PARAM
{
	int32_t 			TestIndex;			/* Test index */
	ADI_DMA_STREAM_ID 	eStreamID;			/* Stream ID */
	MEMTYPE  			eSourceMem;			/* Source memory type (L1/L2/L3) or direct address */
	MEMTYPE 			eDestMem;			/* Destination memory type (L1/L2/L3) or direct address */
	uint32_t			BuffCount;			/* Buffer count in MSIZE unit */
	ADI_DMA_MSIZE		eMsizeCfg;			/* Element width or MSIZE configuration to be used */
	bool				bEnableCallback;	/* Use interrupt of polling method to check for the MDMA completion */
} TEST_PARAM;

typedef struct FAILURE_INFO
{
	uint32_t		TestIndex;
	ADI_DMA_RESULT	ErrorCode;
}FAILURE_INFO;

typedef struct TEST_INFO
{
	TEST_PARAM 		*pTestParamList;
	uint32_t 		FailCount;
	FAILURE_INFO	FailedTestInfo[MAX_TEST_COMBINATIONS];
}TEST_INFO;

volatile int cycle_count;
volatile int cycles_array[1000];
volatile clock_t clock_stop;
volatile clock_t clock_start;

#define START_CYCLE_COUNT\
		cycle_count=0;\
		clock_start = clock();

#define TAKE_CYCLE_SNAPSHOT\
	clock_stop = clock();\
	cycles_array[cycle_count] = clock_stop - clock_start;\
	cycle_count++;\
	clock_start = clock_stop;\

#define PRINT_CYCLES\
	int cc1;\
	for(cc1=0;cc1<cycle_count;cc1++)\
	{ \
		printf("\nCycle count for snapshot %d = %d \n",cc1,cycles_array[cc1]);\
	}


#define PRINT_DRIVER_OVERHEAD\
		int cc, j, k=0,  m;\
		j=cycles_array[1];\
		printf("\nAPI Overhead = %d, ",cycles_array[3]);\
		for(cc=5;cc<cycle_count;cc++)\
		{\
			if(cycles_array[cc]>(j+10))\
			{\
				if(k==0)\
					k=cycles_array[cc];\
			}\
		}\
		m=k-j;\
		printf("Interrupt Overhead = %d \n",m);\

/* Add your custom header content here */

#endif /* __TEST2_MDMA_1DAUTO_1_CORE1_H__ */
